package xyz.wbsite.dbtool.javafx.manger.callable;

import xyz.wbsite.dbtool.javafx.enums.DataBase;
import xyz.wbsite.dbtool.javafx.manger.FreeMarkerManager;
import xyz.wbsite.dbtool.javafx.manger.ManagerFactory;
import xyz.wbsite.dbtool.javafx.manger.ProjectManager;
import xyz.wbsite.dbtool.javafx.manger.XmlManager;
import xyz.wbsite.dbtool.javafx.po.AbstractDBmapper;
import xyz.wbsite.dbtool.javafx.po.Module;
import xyz.wbsite.dbtool.javafx.po.Project;
import xyz.wbsite.dbtool.javafx.po.Table;
import xyz.wbsite.dbtool.javafx.tool.Tool;
import xyz.wbsite.dbtool.web.frame.utils.FileUtil;
import xyz.wbsite.dbtool.web.frame.utils.ResourceUtil;

import java.io.File;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Date;
import java.util.HashMap;
import java.util.concurrent.Callable;

public class SpringBootCallable implements Callable {

    private String root;
    private DataBase dataBase;
    private Project project;
    private String option;
    private boolean cloud;
    private AbstractDBmapper dBmapper;
    private FreeMarkerManager freeMarkerManager;

    public SpringBootCallable(String root, DataBase dataBase, Project project, String option, boolean cloud) {
        this.root = root;
        this.dataBase = dataBase;
        this.project = project;
        this.option = option;
        this.cloud = cloud;
        this.freeMarkerManager = ManagerFactory.getFreeMarkerManager();
        this.dBmapper = ProjectManager.dBmapper;
    }


    public Boolean call() throws Exception {
        // 创建项目文件夹
        File projectDir = Tool.createPath(root, project.getProjectName());
        // 清空内容
        Tool.clear(projectDir);
        // 生成java目录
        File javaDir = Tool.createPath(projectDir.getAbsolutePath(), "src", "main", "java");
        // 生成资源目录
        File resourcesDir = Tool.createPath(projectDir.getAbsolutePath(), "src", "main", "resources");
        // 生成域名目录
        File domainDir = javaDir;
        for (String s : project.getProjectBasePackage().split("\\.")) {
            domainDir = Tool.createPath(domainDir.getAbsolutePath(), s);
        }
        // 生成单元测试java目录
        File testJavaDir = Tool.createPath(projectDir.getAbsolutePath(), "src", "test", "java");
        File testDomainDir = testJavaDir;
        for (String s : project.getProjectBasePackage().split("\\.")) {
            testDomainDir = Tool.createPath(testDomainDir.getAbsolutePath(), s);
        }
        // 生成单元测试resources目录
        File testResourcesDir = Tool.createPath(projectDir.getAbsolutePath(), "src", "test", "resources");

        // 生成POM
        generatePom(projectDir, project, dataBase, option);


        {//生成java文件
            //生成 Controller
            System.out.println("生成模块:action");
            generateController(Tool.createPath(domainDir.getAbsolutePath(), "action"), project, dataBase, option);

            // 生成 Config
            System.out.println("生成模块:config");
            generateConfig(Tool.createPath(domainDir.getAbsolutePath(), "config"), project, dataBase, option);


            {// 模块
                for (Module module : project.getModules()) {// 业务模块
                    module.setProjectAuthor(project.getProjectAuthor());
                    module.setProjectBasePackage(project.getProjectBasePackage());
                    module.setProjectName(project.getProjectName());

                    File moduleDir = Tool.createPath(domainDir.getAbsolutePath(), "module", module.getModuleName());

                    System.out.println("生成模块:Entity");
                    generateEntity(Tool.createPath(moduleDir.getAbsolutePath(), "ent"), module, dataBase, option);
                    System.out.println("生成模块:Mapper");
                    generateMapper(Tool.createPath(moduleDir.getAbsolutePath(), "mpr"), module, dataBase, option);
                    System.out.println("生成模块:Manager");
                    generateManager(Tool.createPath(moduleDir.getAbsolutePath(), "mgr"), module, dataBase, option);
                    System.out.println("生成模块:Request");
                    generateRequest(Tool.createPath(moduleDir.getAbsolutePath(), "req"), module, dataBase, option);
                    System.out.println("生成模块:Response");
                    generateResponse(Tool.createPath(moduleDir.getAbsolutePath(), "rsp"), module, dataBase, option);
                }

                if (project.getSysMD()) {//生成系统模块
                    Module module = new Module();
                    module.setModuleName("system");
                    module.setProjectAuthor(project.getProjectAuthor());
                    module.setProjectBasePackage(project.getProjectBasePackage());
                    module.setProjectName(project.getProjectName());
                    File moduleDir = Tool.createPath(domainDir.getAbsolutePath(), "module", "system");
                    generateSystemModule(moduleDir, module, dataBase, option);
                }
            }

            // frame等内容
            System.out.println("生成模块:frame");
            generateFrame(Tool.createPath(domainDir.getAbsolutePath(), "frame"), project, dataBase, option);

            // Task
            System.out.println("生成模块:task");
            generateTask(Tool.createPath(domainDir.getAbsolutePath(), "task"), project, dataBase, option);

            // Application
            System.out.println("生成模块:Application");
            generateApplication(Tool.createPath(domainDir.getAbsolutePath()), project, dataBase, option);
        }

        {//生成resources文件
            System.out.println("生成模块:Resources");
            generateResources(resourcesDir, project, dataBase, option);
            System.out.println("生成模块:Static");
            generateStatic(resourcesDir, project, dataBase, option);
            System.out.println("生成模块:Templates");
            generateTemplates(resourcesDir, project, dataBase, option);
        }

        {//生成Test
            System.out.println("生成模块:Test");
            generateTest(testDomainDir, project, dataBase, option);
        }

        {//生成Test Resources
            System.out.println("生成模块:Test Resources");
            generateTestResources(testResourcesDir, project, dataBase, option);
        }
        System.out.println("finish");
        return true;
    }


    /**
     * 生成POM.xml
     *
     * @param root
     * @param project
     */
    public void generatePom(File root, Project project, DataBase dataBase, String option) {
        HashMap<String, Object> ctx = new HashMap<String, Object>();
        ctx.put("projectName", project.getProjectName());
        ctx.put("basePackage", project.getProjectBasePackage());
        ctx.put("dataBase", dataBase.toString());
        ctx.put("cloud", cloud);
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "pom.xml"), option + "/pom.ftl", ctx);
    }

    /**
     * 生成Controller类
     *
     * @param root
     * @param project
     */
    public void generateController(File root, Project project, DataBase dataBase, String option) {
        Tool.clear(root);
        HashMap<String, Object> ctx = new HashMap<String, Object>();
        ctx.put("basePackage", project.getProjectBasePackage());
        ctx.put("author", project.getProjectAuthor());
        ctx.put("date", new Date());
        ctx.put("modules", project.getModules());

        File control = Tool.createPath(root.getAbsolutePath(), "control");
        File screen = Tool.createPath(root.getAbsolutePath(), "screen");
        freeMarkerManager.outputTemp(Tool.createFile(control.getAbsolutePath(), "Header.java"), option + "/java/action/control/Header.java", ctx);

        {
            File ajax = Tool.createPath(root.getAbsolutePath(), "ajax");

            for (Module module : project.getModules()) {
                File m = Tool.createPath(ajax.getAbsolutePath(), module.getModuleName());

                for (Table table : module.getTables()) {
                    HashMap<String, Object> ctxss = new HashMap<String, Object>();
                    ctxss.put("basePackage", project.getProjectBasePackage());
                    ctxss.put("module", module.getModuleName());
                    ctxss.put("author", project.getProjectAuthor());
                    ctxss.put("date", new Date());
                    ctxss.put("table", table);

                    freeMarkerManager.outputTemp(Tool.createFile(m.getAbsolutePath(), table.getCName() + "Ajax.java"), option + "/java/action/ajax/Ajax.java", ctxss);
                }
            }

            if (project.getSysMD()) {//生成系统模块
                File system = Tool.createPath(ajax.getAbsolutePath(), "system");

                for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/action/ajax/system/")) {
                    freeMarkerManager.outputTemp(Tool.createFile(system.getAbsolutePath(), name), option + "/java/action/ajax/system/" + name, ctx);
                }
            }
        }

        {
            File api = Tool.createPath(root.getAbsolutePath(), "api");

            for (Module module : project.getModules()) {
                File m = Tool.createPath(api.getAbsolutePath(), module.getModuleName());

                for (Table table : module.getTables()) {
                    HashMap<String, Object> ctxss = new HashMap<String, Object>();
                    ctxss.put("basePackage", project.getProjectBasePackage());
                    ctxss.put("module", module.getModuleName());
                    ctxss.put("author", project.getProjectAuthor());
                    ctxss.put("date", new Date());
                    ctxss.put("table", table);
                    freeMarkerManager.outputTemp(Tool.createFile(m.getAbsolutePath(), table.getCName() + "Api.java"), option + "/java/action/api/Api.java", ctxss);
                }
            }

            if (project.getSysMD()) {//生成系统模块
                File system = Tool.createPath(api.getAbsolutePath(), "system");

                for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/action/api/system/")) {
                    freeMarkerManager.outputTemp(Tool.createFile(system.getAbsolutePath(), name), option + "/java/action/api/system/" + name, ctx);
                }
            }
        }
        freeMarkerManager.outputTemp(Tool.createFile(screen.getAbsolutePath(), "Index.java"), option + "/java/action/screen/Index.java", ctx);
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "GlobalController.java"), option + "/java/action/GlobalController.java", ctx);
    }

    public void generateConfig(File root, Project project, DataBase dataBase, String option) {
        if (!root.exists()) {
            root.mkdirs();
        } else {
            Tool.clear(root);
        }
        HashMap<String, Object> ctx = new HashMap<String, Object>();
        ctx.put("basePackage", project.getProjectBasePackage());
        ctx.put("projectName", project.getProjectName());
        ctx.put("author", project.getProjectAuthor());
        ctx.put("date", new Date());

        //config
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "ActionConfig.java"), option + "/java/config/ActionConfig.java", ctx);
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "CacheConfig.java"), option + "/java/config/CacheConfig.java", ctx);
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "FreeMarkerConfig.java"), option + "/java/config/FreeMarkerConfig.java", ctx);
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "SecurityConfig.java"), option + "/java/config/SecurityConfig.java", ctx);
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "ScheduleConfig.java"), option + "/java/config/ScheduleConfig.java", ctx);
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "ThreadPoolConfig.java"), option + "/java/config/ThreadPoolConfig.java", ctx);
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "WebMvcConfig.java"), option + "/java/config/WebMvcConfig.java", ctx);
        if (dataBase.name().equals(DataBase.SQLite)) {
            freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "SQLiteConfig.java"), option + "/java/config/SQLiteConfig.java", ctx);
        }
        if (project.getMultiDB()) {
            freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "MapperMainConfig.java"), option + "/java/config/MapperMainConfig.java", ctx);
            freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "MapperTwoConfig.java"), option + "/java/config/MapperTwoConfig.java", ctx);
        }
    }

    /**
     * 生成entity
     *
     * @param root
     * @param md
     */
    public void generateEntity(File root, Module md, DataBase dataBase, String option) {
        if (!root.exists()) {
            root.mkdirs();
        } else {
            Tool.clear(root);
        }

        for (Table table : md.getTables()) {
            HashMap<String, Object> ctx = new HashMap<String, Object>();
            ctx.put("tool", Tool.class);
            ctx.put("basePackage", md.getProjectBasePackage());
            ctx.put("moduleName", md.getModuleName());
            ctx.put("table", table);
            ctx.put("author", md.getProjectAuthor());
            ctx.put("date", new Date());

            File file = Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + ".java");
            freeMarkerManager.outputTemp(file, option + "/java/module/ent/entity.ftl", ctx);
        }
    }

    /**
     * 生成Mapper
     *
     * @param root
     * @param md
     */
    public void generateMapper(File root, Module md, DataBase dataBase, String option) {
        if (!root.exists()) {
            root.mkdirs();
        } else {
            Tool.clear(root);
        }
        HashMap<String, Object> ctx = new HashMap<String, Object>();

        ctx.put("tool", Tool.class);
        ctx.put("module", md);
        ctx.put("dataBase", dataBase.toString());
        ctx.put("basePackage", md.getProjectBasePackage());
        ctx.put("moduleName", md.getModuleName());
        ctx.put("author", md.getProjectAuthor());
        ctx.put("date", new Date());

        for (Table table : md.getTables()) {
            ctx.put("table", table);
            freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "Mapper" + ".java"), option + "/java/module/mpr/mapper.java", ctx);

            if (table.getSys()) {
//                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), dataBase.name() + "_" + Tool.lineToClassName(table.getTableName()) + "Mapper" + ".xml"), option + "/java/module/mpr/" + dataBase.name() + "_mapper.xml", ctx);
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "Mapper" + ".xml"), option + "/java/module/mpr/" + dataBase.name() + "_mapper.xml", ctx);
            } else {
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "Mapper" + ".xml"), option + "/java/module/mpr/Nosys_mapper.ftl", ctx);
            }
        }
    }

    public void generateManager(File root, Module md, DataBase dataBase, String option) {
        if (!root.exists()) {
            root.mkdirs();
        } else {
            Tool.clear(root);
        }
        HashMap<String, Object> ctx = new HashMap<String, Object>();

        ctx.put("tool", Tool.class);
        ctx.put("module", md);
        ctx.put("basePackage", md.getProjectBasePackage());
        ctx.put("moduleName", md.getModuleName());
        ctx.put("author", md.getProjectAuthor());
        ctx.put("date", new Date());

        for (Table table : md.getTables()) {
            ctx.put("table", table);
            freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "Manager" + ".java"), option + "/java/module/mgr/manager.ftl", ctx);
            freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "ManagerImpl" + ".java"), option + "/java/module/mgr/managerImpl.ftl", ctx);
        }
    }

    /**
     * 生成Request
     *
     * @param root
     * @param md
     */
    public void generateRequest(File root, Module md, DataBase dataBase, String option) {
        if (!root.exists()) {
            root.mkdirs();
        } else {
            Tool.clear(root);
        }

        HashMap<String, Object> ctx = new HashMap<String, Object>();
        ctx.put("tool", Tool.class);
        ctx.put("module", md);
        ctx.put("basePackage", md.getProjectBasePackage());
        ctx.put("moduleName", md.getModuleName());
        ctx.put("author", md.getProjectAuthor());
        ctx.put("date", new Date());

        if (md.getModuleName().equals("api")) {
            freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "ApiExampleRequest" + ".java"), option + "/java/module/req/ApiExampleRequest.java", ctx);
        }

        for (Table table : md.getTables()) {
            ctx.put("table", table);

            if (table.getCreate()) {
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "CreateRequest" + ".java"), option + "/java/module/req/createRequestClass.ftl", ctx);
            }

            if (table.getDelete()) {
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "DeleteRequest" + ".java"), option + "/java/module/req/deleteRequestClass.ftl", ctx);
            }

            if (table.getUpdate()) {
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "UpdateRequest" + ".java"), option + "/java/module/req/updateRequestClass.ftl", ctx);
            }

            if (table.getFind()) {
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "FindRequest" + ".java"), option + "/java/module/req/findRequestClass.ftl", ctx);
            }

            if (table.getGet()) {
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "GetRequest" + ".java"), option + "/java/module/req/getRequestClass.ftl", ctx);
            }

            if (table.getSearch()) {
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "SearchRequest" + ".java"), option + "/java/module/req/searchRequestClass.ftl", ctx);
            }
        }
    }

    /**
     * 生成一般文件
     *
     * @param root
     * @param md
     */
    public void generateResponse(File root, Module md, DataBase dataBase, String option) {
        if (!root.exists()) {
            root.mkdirs();
        } else {
            Tool.clear(root);
        }

        HashMap<String, Object> ctx = new HashMap<String, Object>();
        ctx.put("tool", Tool.class);
        ctx.put("module", md);
        ctx.put("basePackage", md.getProjectBasePackage());
        ctx.put("moduleName", md.getModuleName());
        ctx.put("author", md.getProjectAuthor());
        ctx.put("date", new Date());

        if (md.getModuleName().equals("api")) {
            freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "ApiExampleResponse" + ".java"), option + "/java/module/rsp/ApiExampleResponse.java", ctx);
        }

        for (Table table : md.getTables()) {
            ctx.put("table", table);

            if (table.getCreate()) {
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "CreateResponse" + ".java"), option + "/java/module/rsp/createResponseClass.ftl", ctx);
            }

            if (table.getDelete()) {
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "DeleteResponse" + ".java"), option + "/java/module/rsp/deleteResponseClass.ftl", ctx);
            }

            if (table.getUpdate()) {
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "UpdateResponse" + ".java"), option + "/java/module/rsp/updateResponseClass.ftl", ctx);
            }

            if (table.getFind()) {
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "FindResponse" + ".java"), option + "/java/module/rsp/findResponseClass.ftl", ctx);
            }

            if (table.getGet()) {
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "GetResponse" + ".java"), option + "/java/module/rsp/getResponseClass.ftl", ctx);
            }

            if (table.getSearch()) {
                freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "SearchResponse" + ".java"), option + "/java/module/rsp/searchResponseClass.ftl", ctx);
            }
        }
    }

    public void generateSystemModule(File root, Module md, DataBase dataBase, String option) {
        File ent = Tool.createPath(root.getAbsolutePath(), "ent");
        File mpr = Tool.createPath(root.getAbsolutePath(), "mpr");
        File mgr = Tool.createPath(root.getAbsolutePath(), "mgr");
        File req = Tool.createPath(root.getAbsolutePath(), "req");
        File rsp = Tool.createPath(root.getAbsolutePath(), "rsp");

        HashMap<String, Object> ctx = new HashMap<String, Object>();
        ctx.put("basePackage", md.getProjectBasePackage());
        ctx.put("moduleName", md.getModuleName());
        ctx.put("author", md.getProjectAuthor());
        ctx.put("date", new Date());
        ctx.put("dataBase", dataBase.toString());

        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/module/system/ent/")) {
            freeMarkerManager.outputTemp(Tool.createFile(ent.getAbsolutePath(), name), option + "/java/module/system/ent/" + name, ctx);
        }

        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/module/system/mgr/")) {
            freeMarkerManager.outputTemp(Tool.createFile(mgr.getAbsolutePath(), name), option + "/java/module/system/mgr/" + name, ctx);
        }

        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/module/system/mpr/")) {
            if (name.endsWith(".java")) {
                freeMarkerManager.outputTemp(Tool.createFile(mpr.getAbsolutePath(), name), option + "/java/module/system/mpr/" + name, ctx);
            } else if (name.contains(dataBase.name())) {
                //todo dateBase调试
//                freeMarkerManager.outputTemp(Tool.createFile(mpr.getAbsolutePath(), name), option + "/java/module/system/mpr/" + name, ctx);
                freeMarkerManager.outputTemp(Tool.createFile(mpr.getAbsolutePath(), name.replaceAll(dataBase.name() + "_", "")), option + "/java/module/system/mpr/" + name, ctx);
            }
        }

        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/module/system/req/")) {
            freeMarkerManager.outputTemp(Tool.createFile(req.getAbsolutePath(), name), option + "/java/module/system/req/" + name, ctx);
        }

        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/module/system/rsp/")) {
            freeMarkerManager.outputTemp(Tool.createFile(rsp.getAbsolutePath(), name), option + "/java/module/system/rsp/" + name, ctx);
        }
    }

    /**
     * 生成frame类
     *
     * @param root
     * @param project
     */
    public void generateFrame(File root, Project project, DataBase dataBase, String option) {
        if (!root.exists()) {
            root.mkdirs();
        } else {
            Tool.clear(root);
        }
        HashMap<String, Object> ctx = new HashMap<String, Object>();
        ctx.put("basePackage", project.getProjectBasePackage());
        ctx.put("moduleName", project.getProjectName());
        ctx.put("timestamp", new Date().getTime());

        File auth = Tool.createPath(root.getAbsolutePath(), "auth");
        File base = Tool.createPath(root.getAbsolutePath(), "base");
        File excel = Tool.createPath(root.getAbsolutePath(), "excel");
        File excelannotation = Tool.createPath(excel.getAbsolutePath(), "annotation");
        File excelconverter = Tool.createPath(excel.getAbsolutePath(), "converter");
        File exception = Tool.createPath(excel.getAbsolutePath(), "exception");
        File excelstyle = Tool.createPath(excel.getAbsolutePath(), "style");
        File schedule = Tool.createPath(root.getAbsolutePath(), "schedule");
        File utils = Tool.createPath(root.getAbsolutePath(), "utils");
        File validation = Tool.createPath(root.getAbsolutePath(), "validation");

        //auth
        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/frame/auth/")) {
            freeMarkerManager.outputTemp(Tool.createFile(auth.getAbsolutePath(), name), option + "/java/frame/auth/" + name, ctx);
        }

        //base
        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/frame/base/")) {
            freeMarkerManager.outputTemp(Tool.createFile(base.getAbsolutePath(), name), option + "/java/frame/base/" + name, ctx);
        }

        //excel
        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/frame/excel/")) {
            freeMarkerManager.outputTemp(Tool.createFile(excel.getAbsolutePath(), name), option + "/java/frame/excel/" + name, ctx);
        }
        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/frame/excel/annotation/")) {
            freeMarkerManager.outputTemp(Tool.createFile(excelannotation.getAbsolutePath(), name), option + "/java/frame/excel/annotation/" + name, ctx);
        }
        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/frame/excel/converter/")) {
            freeMarkerManager.outputTemp(Tool.createFile(excelconverter.getAbsolutePath(), name), option + "/java/frame/excel/converter/" + name, ctx);
        }
        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/frame/excel/exception/")) {
            freeMarkerManager.outputTemp(Tool.createFile(exception.getAbsolutePath(), name), option + "/java/frame/excel/exception/" + name, ctx);
        }
        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/frame/excel/style/")) {
            freeMarkerManager.outputTemp(Tool.createFile(excelstyle.getAbsolutePath(), name), option + "/java/frame/excel/style/" + name, ctx);
        }

        //schedule
        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/frame/schedule/")) {
            freeMarkerManager.outputTemp(Tool.createFile(schedule.getAbsolutePath(), name), option + "/java/frame/schedule/" + name, ctx);
        }
        //utils
        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/frame/utils/")) {
            freeMarkerManager.outputTemp(Tool.createFile(utils.getAbsolutePath(), name), option + "/java/frame/utils/" + name, ctx);
        }

        //validation
        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/frame/validation/")) {
            freeMarkerManager.outputTemp(Tool.createFile(validation.getAbsolutePath(), name), option + "/java/frame/validation/" + name, ctx);
        }
    }


    /**
     * 生成frame类
     *
     * @param root
     * @param project
     */
    public void generateTask(File root, Project project, DataBase dataBase, String option) {
        if (!root.exists()) {
            root.mkdirs();
        } else {
            Tool.clear(root);
        }
        HashMap<String, Object> ctx = new HashMap<String, Object>();
        ctx.put("basePackage", project.getProjectBasePackage());
        ctx.put("moduleName", project.getProjectName());
        ctx.put("timestamp", new Date().getTime());

        //task
        for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/java/task/")) {
            freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), name), option + "/java/task/" + name, ctx);
        }
    }

    /**
     * 生成base类
     *
     * @param root
     * @param project
     */
    public void generateApplication(File root, Project project, DataBase dataBase, String option) {

        HashMap<String, Object> ctx = new HashMap<String, Object>();
        ctx.put("basePackage", project.getProjectBasePackage());
        ctx.put("moduleName", project.getProjectName());
        ctx.put("multiDB", project.getMultiDB());
        ctx.put("timestamp", new Date().getTime());

        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "Application.java"), option + "/java/Application.ftl", ctx);
    }

    /**
     * 生成properties
     *
     * @param root
     * @param project
     */
    public void generateResources(File root, Project project, DataBase dataBase, String option) {
        HashMap<String, Object> ctx = new HashMap<String, Object>();

        ctx.put("basePackage", project.getProjectBasePackage());
        ctx.put("tool", Tool.class);
        ctx.put("dataBase", dataBase.toString());
        ctx.put("author", project.getProjectAuthor());
        ctx.put("dBmapper", dBmapper);
        ctx.put("date", new Date());
        ctx.put("author", project.getProjectAuthor());
        ctx.put("projectName", project.getProjectName());
        ctx.put("multiDB", project.getMultiDB());
        ctx.put("cloud", cloud);
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "start-dev.bat"), option + "/resources/start-dev.bat", ctx);
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "start-prod.bat"), option + "/resources/start-prod.bat", ctx);
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "application-dev.properties"), option + "/resources/application-dev.ftl", ctx);
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "application-prod.properties"), option + "/resources/application-prod.ftl", ctx);
        Tool.outputResource(option + "/resources/logback-spring.xml", Tool.createFile(root.getAbsolutePath(), "logback-spring.xml"));

        File dbtool = Tool.createPath(root.getAbsolutePath(), "dbtool");
        {
            XmlManager xmlManager = ManagerFactory.getXmlManager();
            xmlManager.saveAs(dbtool, project);
        }

        for (Module module : project.getModules()) {
            ctx.put("moduleName", module.getModuleName());
            ctx.put("module", module);
            File tableDir = Tool.createPath(dbtool.getAbsolutePath(), module.getModuleName() + "_table");
            for (Table table : module.getTables()) {
                ctx.put("table", table);
                freeMarkerManager.outputTemp(Tool.createFile(tableDir.getAbsolutePath(), dataBase.name() + "_" + table.getTableName() + ".sql"), option + "/resources/dbtool/table.ftl", ctx);
            }
            freeMarkerManager.outputTemp(Tool.createFile(tableDir.getAbsolutePath(), dataBase.name() + "_ALL_TABLE" + ".sql"), option + "/resources/dbtool/tableAll.ftl", ctx);
        }

        if (project.getSysMD()) {//生成系统模块
            File system = Tool.createPath(dbtool.getAbsolutePath(), "system_table");

            for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/resources/dbtool/system/")) {
                if (name.contains(dataBase.name())) {
                    freeMarkerManager.outputTemp(Tool.createFile(system.getAbsolutePath(), name), option + "/resources/dbtool/system/" + name, ctx);
                }
            }
        }

        if (DataBase.Oracle.name().equals(dataBase.name())) {
            File lib = Tool.createPath(root.getAbsolutePath(), "lib");
            Tool.outputResource(option + "/resources/lib/ojdbc7-12.1.0.2.jar", Tool.createFile(lib.getAbsolutePath(), "ojdbc7-12.1.0.2.jar"));
        }

        if (DataBase.SQLite.name().equals(dataBase.name())) {
            File dbFile = Tool.createFile(root.getAbsolutePath(), project.getProjectName() + ".db");
            Connection connection = null;
            Statement statement = null;
            try {
                Class.forName("org.sqlite.JDBC");
                connection = DriverManager.getConnection("jdbc:sqlite:" + dbFile.getAbsolutePath());

                for (File fileModule : dbtool.listFiles()) {
                    if (!fileModule.isDirectory()) {
                        continue;
                    }
                    for (File file : fileModule.listFiles()) {
                        if (!file.getName().startsWith("ALL_") && file.getName().endsWith(".sql")) {
                            statement = connection.createStatement();
                            String sql = FileUtil.readFileToString(file);
                            statement.execute(sql);
                            statement.close();
                        }
                    }
                }
            } catch (ClassNotFoundException | SQLException | IOException e) {
                e.printStackTrace();
            } finally {
                try {
                    if (statement != null) statement.close();
                    if (connection != null) connection.close();
                } catch (Exception e) {

                }
            }
        }
    }

    public void generateTestResources(File root, Project project, DataBase dataBase, String option) {
        HashMap<String, Object> ctx = new HashMap<String, Object>();

        ctx.put("basePackage", project.getProjectBasePackage());
        ctx.put("projectName", project.getProjectName());
        ctx.put("tool", Tool.class);
        ctx.put("dataBase", dataBase.toString());
        ctx.put("author", project.getProjectAuthor());
        ctx.put("dBmapper", dBmapper);
        ctx.put("date", new Date());
        ctx.put("multiDB", project.getMultiDB());
        ctx.put("cloud", cloud);
        freeMarkerManager.outputTemp(Tool.createFile(root.getAbsolutePath(), "application.properties"), option + "/test/application.ftl", ctx);
        Tool.outputResource(option + "/test/logback-spring.xml", Tool.createFile(root.getAbsolutePath(), "logback-spring.xml"));
    }

    /**
     * 生成static
     *
     * @param root
     * @param project
     */
    public void generateStatic(File root, Project project, DataBase dataBase, String option) {

        HashMap<String, Object> ctx = new HashMap<String, Object>();
        ctx.put("basePackage", project.getProjectBasePackage());
        ctx.put("tool", Tool.class);
        ctx.put("author", project.getProjectAuthor());
        ctx.put("date", new Date());
        ctx.put("modules", project.getModules());

        File static_ = Tool.createPath(root.getAbsolutePath(), "static");
        File css = Tool.createPath(static_.getAbsolutePath(), "css");
        File img = Tool.createPath(static_.getAbsolutePath(), "img");
        File dist = Tool.createPath(static_.getAbsolutePath(), "dist");

        {
            Tool.outputResource(option + "/resources/static/favicon.ico", Tool.createFile(static_.getAbsolutePath(), "favicon.ico"));
        }

        {//css文件
            freeMarkerManager.outputTemp(Tool.createFile(css.getAbsolutePath(), "base.css"), option + "/resources/static/css/base.css", ctx);
        }

        {//img
            Tool.outputResource(option + "/resources/static/img/logo.png", Tool.createFile(img.getAbsolutePath(), "logo.png"));
        }

        {//dist文件
            //jquery + vue + vue-router + axios
            Tool.outputResource(option + "/resources/static/dist/lib.min.js", Tool.createFile(dist.getAbsolutePath(), "lib.min.js"));

            //element-ui
            Tool.outputResource(option + "/resources/static/dist/index.min.js", Tool.createFile(dist.getAbsolutePath(), "index.min.js"));
            Tool.outputResource(option + "/resources/static/dist/index.min.css", Tool.createFile(dist.getAbsolutePath(), "index.min.css"));

            File fonts = Tool.createPath(dist.getAbsolutePath(), "fonts");
            Tool.outputResource(option + "/resources/static/dist/fonts/w-e-icon.woff", Tool.createFile(fonts.getAbsolutePath(), "w-e-icon.woff"));
            Tool.outputResource(option + "/resources/static/dist/fonts/element-icons.woff", Tool.createFile(fonts.getAbsolutePath(), "element-icons.woff"));
            Tool.outputResource(option + "/resources/static/dist/fonts/element-icons.ttf", Tool.createFile(fonts.getAbsolutePath(), "element-icons.ttf"));

        }
    }

    /**
     * 生成static
     *
     * @param root
     * @param project
     */
    public void generateTemplates(File root, Project project, DataBase dataBase, String option) {
        {//生成
            File templates = Tool.createPath(root.getAbsolutePath(), "templates");
            File control = Tool.createPath(templates.getAbsolutePath(), "control");
            File layout = Tool.createPath(templates.getAbsolutePath(), "layout");
            File screen = Tool.createPath(templates.getAbsolutePath(), "screen");
            Tool.outputResource(option + "/resources/templates/control/nav.ftl", Tool.createFile(control.getAbsolutePath(), "nav.ftl"));
            Tool.outputResource(option + "/resources/templates/control/macro.ftl", Tool.createFile(control.getAbsolutePath(), "macro.ftl"));
            Tool.outputResource(option + "/resources/templates/layout/default.ftl", Tool.createFile(layout.getAbsolutePath(), "default.ftl"));
            Tool.outputResource(option + "/resources/templates/screen/home.ftl", Tool.createFile(screen.getAbsolutePath(), "home.ftl"));
            Tool.outputResource(option + "/resources/templates/screen/login.ftl", Tool.createFile(screen.getAbsolutePath(), "login.ftl"));
            Tool.outputResource(option + "/resources/templates/403.ftl", Tool.createFile(templates.getAbsolutePath(), "403.ftl"));
            Tool.outputResource(option + "/resources/templates/404.ftl", Tool.createFile(templates.getAbsolutePath(), "404.ftl"));
            Tool.outputResource(option + "/resources/templates/500.ftl", Tool.createFile(templates.getAbsolutePath(), "500.ftl"));

            {
                HashMap<String, Object> ctx = new HashMap<String, Object>();
                ctx.put("basePackage", project.getProjectBasePackage());
                ctx.put("tool", new Tool());
                ctx.put("author", project.getProjectAuthor());
                ctx.put("date", new Date());
                ctx.put("modules", project.getModules());
                ctx.put("sysMD", project.getSysMD());
                freeMarkerManager.outputTemp(Tool.createFile(screen.getAbsolutePath(), "index.ftl"), option + "/resources/templates/screen/index.ftl", ctx);

            }

            for (Module mo : project.getModules()) {
                if (mo.isHasSysFields()) {//标准模型才好生成简单的htm
                    for (Table table : mo.getTables()) {
                        if (table.getHtml()) {
                            File module = Tool.createPath(screen.getAbsolutePath(), mo.getModuleName());
                            HashMap<String, Object> ctx = new HashMap<String, Object>();
                            ctx.put("basePackage", project.getProjectBasePackage());
                            ctx.put("tool", Tool.class);
                            ctx.put("author", project.getProjectAuthor());
                            ctx.put("date", new Date());
                            ctx.put("table", table);
                            ctx.put("module", module.getName());
                            ctx.put("fields", table.getFields());
                            File file = Tool.createFile(module.getAbsolutePath(), table.getFName() + ".ftl");
                            freeMarkerManager.outputTemp(file, option + "/resources/templates/screen/module/mgr.ftl", ctx);
                        }
                    }
                }
            }

            if (project.getSysMD()) {//生成系统模块
                File system = Tool.createPath(screen.getAbsolutePath(), "system");
                for (String name : ResourceUtil.getResourceFiles("/modules/SpringBoot/resources/templates/screen/module/system/")) {
                    Tool.outputResource(option + "/resources/templates/screen/module/system/" + name, Tool.createFile(system.getAbsolutePath(), name));
                }
            }
        }
    }

    public void generateTest(File root, Project project, DataBase dataBase, String option) {

        {
            File config = Tool.createPath(root.getAbsolutePath(), "config");
            config.mkdirs();
            HashMap<String, Object> ctx = new HashMap<String, Object>();
            ctx.put("tool", Tool.class);
            ctx.put("basePackage", project.getProjectBasePackage());
            ctx.put("moduleName", project.getProjectName());
            ctx.put("author", project.getProjectAuthor());
            ctx.put("date", new Date());
            File file = Tool.createFile(config.getAbsolutePath(), "TestConfig" + ".java");
            freeMarkerManager.outputTemp(file, option + "/test/TestConfig.ftl", ctx);
        }

        {
            HashMap<String, Object> ctx = new HashMap<String, Object>();
            ctx.put("tool", Tool.class);
            ctx.put("basePackage", project.getProjectBasePackage());
            ctx.put("moduleName", project.getProjectName());
            ctx.put("author", project.getProjectAuthor());
            ctx.put("date", new Date());
            File file = Tool.createFile(root.getAbsolutePath(), "UtilTest" + ".java");
            freeMarkerManager.outputTemp(file, option + "/test/UtilTest.java", ctx);
        }

        for (Module md : project.getModules()) {
            File module = Tool.createPath(root.getAbsolutePath(), md.getModuleName());

            for (Table table : md.getTables()) {
                HashMap<String, Object> ctx = new HashMap<String, Object>();

                ctx.put("tool", Tool.class);
                ctx.put("module", md);
                ctx.put("basePackage", md.getProjectBasePackage());
                ctx.put("moduleName", md.getModuleName());
                ctx.put("table", table);
                ctx.put("author", md.getProjectAuthor());
                ctx.put("date", new Date());

                File file = Tool.createFile(module.getAbsolutePath(), Tool.lineToClassName(table.getTableName()) + "Test" + ".java");
                freeMarkerManager.outputTemp(file, option + "/test/test.ftl", ctx);
            }
        }

        if (project.getSysMD()) {//生成系统模块
            HashMap<String, Object> ctx = new HashMap<String, Object>();
            ctx.put("tool", Tool.class);
            ctx.put("basePackage", project.getProjectBasePackage());
            ctx.put("moduleName", project.getProjectName());
            ctx.put("author", project.getProjectAuthor());
            ctx.put("date", new Date());
            File system = Tool.createPath(root.getAbsolutePath(), "system");

            for (String apiFile : ResourceUtil.getResourceFiles("/modules/SpringBoot/test/system/")) {
                freeMarkerManager.outputTemp(Tool.createFile(system.getAbsolutePath(), apiFile), option + "/test/system/" + apiFile, ctx);
            }
        }
    }
}
